"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.yargsAddGlobalOptions = exports.execShellInteractive = exports.execShellToString = exports.generateHookScript = exports.substVars = exports.env = void 0;
const child_process_1 = require("child_process");
function env(content, variables) {
    // https://regex101.com/r/k9saS6/2
    // Yes:
    //  ${NAME:DEFAULT}
    //  ${NAME:"DEFAULT"}
    //  ${NAME}
    // Not:
    //  ${NAME:}
    const R = /\$\{([A-Z0-9_]+(\:[^\}]+)?)\}/gi;
    return content.replace(R, (_, result) => {
        let [name, value, ...rest] = result.split(':');
        if (value) {
            if (rest && rest.length) {
                value = [value, ...rest].join(':');
            }
            value = value.trim();
            if (value.startsWith('$')) {
                value = variables[value.replace('$', '')] || '';
            }
            else if (value.startsWith(`"`)) {
                value = value.replace(/^\"([^\"]+)\"$/g, '$1');
            }
            else if (value.startsWith(`'`)) {
                value = value.replace(/^\'([^\']+)\'$/g, '$1');
            }
        }
        return variables[name] ? String(variables[name]) : value;
    });
}
exports.env = env;
const substVars = (expr, ctx) => {
    const ctxDict = {};
    for (const [k, v] of ctx) {
        ctxDict[k] = v;
    }
    return env(expr, ctxDict);
};
exports.substVars = substVars;
const generateHookScript = (scripts, uniformBinary) => {
    const result = [];
    result.push('#!/bin/bash');
    result.push('set -e');
    result.push(`printf "\x1b[0;34m%s\x1b[39;49;00m\n" "Run hook in Uniform CLI"`);
    for (const script of scripts) {
        result.push(`${uniformBinary} --mode=hook --no-tty ${script}`);
    }
    return result.join('\n');
};
exports.generateHookScript = generateHookScript;
/**
 * Executes a shell command and return it as a Promise.
 */
async function execShellToString(cmd, cwd, env) {
    return new Promise((resolve, reject) => {
        (0, child_process_1.exec)(cmd, { env: { ...process.env, ...env }, cwd }, (error, stdout) => {
            if (error) {
                reject(error);
            }
            else {
                resolve(stdout);
            }
        });
    });
}
exports.execShellToString = execShellToString;
/**
 * Executes a shell command, sends all data to stdout and stderr.
 */
async function execShellInteractive(cmd, cwd, env) {
    return new Promise((resolve, reject) => {
        const shell = (0, child_process_1.spawn)(cmd, {
            stdio: 'inherit',
            shell: true,
            cwd,
            env: { ...process.env, ...env },
        });
        shell.on('close', code => resolve(code));
        shell.on('error', err => reject(err));
    });
}
exports.execShellInteractive = execShellInteractive;
const yargsAddGlobalOptions = (yargs) => yargs
    .option('debug', {
    type: 'boolean',
    default: false,
    description: 'print debug messages',
})
    .option('dry-run', {
    alias: 'dryRun',
    default: false,
    type: 'boolean',
    description: 'do not execute a real command, only print',
})
    .option('uid', {
    type: 'number',
    default: 0,
    description: 'use another uid, by default uses uid of current user',
})
    .option('no-tty', {
    alias: 'noTty',
    type: 'boolean',
    default: false,
    description: 'disable pseudo-TTY allocation',
});
exports.yargsAddGlobalOptions = yargsAddGlobalOptions;
